\ For noForth C&V2553 lp.0, Low Power Mode, RS232 interrupt using machine code.

( This is an enhancement from previous code.
  This are the main routines:
    - SLEEP     goto deep sleep LMP4 I~0,0006mA and activate RX-interrupt
    - WAKE-RX   Wakeup when a new key arrives
    - WAKE-TX   Wakeup when emit is free to use
    - KEY}      Goto sleep when no KEY was received
    - EMIT}     Goto sleep when previous EMIT is still busy
    - MS1       A low power variant of MS uses the LPM0 I~0,35mA

    SLEEP must be placed before ACCEPT for example in REFILL
    SLEEP-ON must be added to the cold start code.
    KEY} and EMIT} are the new i/o vectors.
    Right before .OK the leds must be put on, right there after
    SLEEP must be placed, it also puts the leds off!

    Bit-3 in the OK flag register is used for setting the
    sleep mode noForth is in to. A zero means minimal use of
    sleep modes, noForth only uses LPM0 where most clocks keep running.
    When bit-3 is one noForth uses deeper sleep modes. Like going
    into deep sleep before ACCEPT and using LPM2 to further save current.
 )

( MSP430 power modes name shortcuts & short explanation:
  AM   = Active Mode      - 00 - CPU & clocks active
  LPM0 = Low Power Mode 0 - 10 - CPU & MCLK disabled, DCO, SMCLK & ACLK active
  LPM1 = Low Power Mode 1 - 50 - Same as mode 0 except when DCO is not used
  LPM2 = Low Power Mode 2 - 90 - Only DC generator & ACLK active
  LPM3 = Low Power Mode 3 - D0 - Only ACLK active
  LPM4 = Low Power Mode 4 - F0 - CPU & clocks disabled
  More info at pages 41, etc. and page 46 of SLAU144J.PDF 

  0021 - P1OUT          - Port-1 Ouput reg.
  0001 - IE2    Bit-0,1 - Interrupt Enable reg. 2
  0003 - IFG2   Bit-0,1 - Interrupt Flag reg. 2
  FFEE/FFEC             - RS232 RX/TX-interrupt vector
 )

( About the average current use.

 Standard noForth uses about 10,5mA, without leds that is 2,6mA.
 The first current number is while the system waits for a key, 
 the second current number is the average use while running.

OK-contents:      OK=0F            OK=0F                OK=07         Not relevant
 - DCO freq.  - LPM4-sleep  -      LPM2        -        LPM0         - Now in AM
    1 MHz        0,0006mA     0,022mA/0,175mA       0,070mA/0,18mA      0,35mA
    8 MHz        0,0006mA     0,025mA/0,480mA       0,345mA/0,50mA      2,60mA
   16 MHz        0,0006mA     0,033mA/0,820mA       0,682mA/0,83mA      5,20mA

When bit 3 in OK is set, we are in LPM2 otherwise we are in LPM0
At the start of ACCEPT we are in LPM4, but only when bit 3 in OK is set.

The red & green led are used as visual prompt, the green signals an
OK, the red led an error condition. 
Note that a vector XT for the wanted clock is preferable! 
Default it contains the word 8MHZ
 )

hex
code LPM0     ( -- )    18 # sr bis  next  end-code \ Go from AM to LPM0
code INT-OFF  ( -- )    #8 sr bic  next  end-code
code INT-ON   ( -- )    #8 sr bis  next  end-code
code ?S       ( -- +n ) tos sp -) mov  sr tos mov  next end-code  

\ Switch DCO to 1 MHz, current use of CPU without leds ~ 0,35mA
: 1MHZ      ( -- )  \ Baudrate 9600b
    int-off  2 ms
    0060 0056 C!    \ DCOCTL    Set DCO to lowest 
    10FF c@ 0057 c! \ BCSCTL1   Set DCO = 1MHz
    10FE c@ 0056 c! \ DCOCTL
    0000 0058 c!    \ UCA0CTL2  DCO on
    0081 0061 *bis  \ UCA0CTL1  Uart use SMCLK
    0068 0062 c!    \ UCA0BR0   1 mhz, baudrate 9600
    0000 0063 c!    \ UCA0BR1   idem
    0002 0064 c!    \ UCA0CTL0  modulation ucbrsx = 1
    0001 0061 *bic  \ UCA0CTL   enable USCI
    A542 012A !     \ FCTL2     mclk/3=333KHz
( ) 0038 TO MS) ;   \ MS timing

\ Set DCO to 8 MHz, current use of CPU full running without leds ~ 2,56mA
: 8MHZ      ( -- )  \ Baudrate 9600B
    int-off  2 ms
    0060 0056 C!    \ DCOCTL    Set DCO to lowest 
    10FD c@ 057 c!  \ BCSCTL1   set dco = 8 mhz
    10FC c@ 056 c!  \ DCOCTL
    0000 058 c!     \ UCA0CTL2  DCO on
    0081 061 *bis   \ UCA0CTL1  Uart use SMCLK
    0041 062 c!     \ UCA0BR0   8 mhz, baudrate 9600
    0003 063 c!     \ UCA0BR1   idem
    0004 064 c!     \ UCAMCTL   modulation ucbrsx = 2
    0001 061 *bic   \ UCA0CTL   enable USCI
    A550 012A !     \ FCTL2     mclk/17=470KHz flash timing
( ) 01C0 TO MS) ;   \ MS timing

code SLEEP      ( -- )
    41 # 021 & .b bic       \ Leds off
    #8  adr ok & bit        \ Low power mode?
    <>? if,
        #1 001 & .b bis     \ RS232 RX interrupt on
        F8 # sr bis         \ Enter LPF4 & int. on (0,6uA)
    then,
    next
end-code

value kin  value kuit  value flags
code KEY}       ( -- c )
    tos sp -) mov           \ Push TOS
    #1 003 & .b bit         \ Test RX flag
    =? if,                  \ Not set ?
        #1 001 & .b bis
        #8  adr ok & bit    \ Which power mode?
        <>? if,
            98 # sr bis     \ Go from AM to LPM2
        else,
            18 # sr bis     \ Go from AM to LPM0
        then,
        adr kin & tos .b mov
        next
    then,
    066 &  tos .b mov       \ Get char
    next
end-code

code EMIT}      ( c -- )    
    #2 003 & .b bit         \ Test TX flag
    =? if,                  \ Not set ?
        #2 001 & .b bis
        tos  adr kuit & .b mov  \ Send char
        #8  adr ok & bit    \ Which power mode
        <>? if,
            98 # sr bis     \ Go from AM to LPM2
        else,
            18 # sr bis     \ Go from AM to LPM0
        then,
        sp )+ tos mov       \ Pop tos
        next
    then,
    tos  067 & .b mov       \ Send char
    sp )+ tos mov           \ Pop tos
    next
end-code

\ Wakeup when KEY was received and exit low power mode
code WAKE-RX    ( -- )
    #1 001 & .b bic         \ RS232 RX off
    066 & adr kin & .b mov
    F0 # rp ) bic           \ LPM off
    reti
end-code

\ Wakeup when EMIT was transmitted and exit low power mode
code WAKE-TX    ( -- )
    #2 001 & .b bic         \ RS232 TX off
    adr kuit & 67 & .b mov
    F0 # rp ) bic           \ LPM off
   reti
end-code


\ MS with watchdog as interval timer
value MS)
code MSTIMER    ( -- )  \ Decrease (MS) until it's zero
    #0  adr ms) & cmp
    =? if,
        #1 000 & .b bic \ Watchdog int. off
        F0 # rp ) bic   \ LPM off
        reti
    then,
    #1 adr ms) & sub
    reti
end-code

\ An MS routine using the Watchdog interval mode in LPM0,
\ This could go as far as LPM3 but noForth stops in LPM3.
\ And this is not desirable.
: (MS)          ( u x -- )
    120 !  to ms)       \ Activate watchdog as MS-timer
    01 000 *bis  lpm0   \ Go to sleep with interrupt on
    5A91 120 ! ;        \ Stop watchdog

\ This variant should always be correct 
: MS1           ( u -- )
    012A c@  ( Use  flash timing as reference )
    dup 00 = if  drop  1 lshift 5A1B (ms) exit  then       \ 32 KHz
    dup 42 = if  drop  dup 5A1A (ms)  5A1A (ms) exit  then \ 1 MHz
        62 = if  dup 5A19 (ms)  then                       \ 16 MHz
    5A19 (ms) ;                                            \ 8 MHz

: SLEEP-ON      ( -- )  \ Set LPM RS232 variant routines
( ) int-off
( ) F7 022 *bis         \ Only P1.3 is input
( ) 41 021 *bic         \ Leds off
( ) FF 029 *bic         \ Set P2 low
( ) ['] emit} to 'emit  \ Replace vectors
( ) ['] key} to 'key 
( ) 5A91 120 ! ;        \ Stop watchdog

: SLEEP-OFF     ( -- )  \ Restore old RS232 routines
    int-off
    ['] emit) to 'emit
    ['] key) to 'key ;

' wake-tx >body  FFEC vec!  \ TX intrpt.
' wake-rx >body  FFEE vec!  \ RX intrpt.
' mstimer >body  FFF4 vec!  \ Watchdog intrpt.

shield SLEEP\  
' sleep-on to app  freeze

\ End
